/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.remoting.transport.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Map;
import org.jboss.remoting.InvokerLocator;
import org.jboss.logging.Logger;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class ClientSocketWrapper extends SocketWrapper
{
   private static final Logger log = Logger.getLogger(ClientSocketWrapper.class);

   private static boolean trace = log.isTraceEnabled();

   private InputStream in;
   private OutputStream out;

   public ClientSocketWrapper(Socket socket) throws IOException
   {
      super(socket);
      createStreams(socket, null);
   }

   public ClientSocketWrapper(Socket socket, Map metadata, Integer timeout) throws Exception
   {
      super(socket, timeout);
      createStreams(socket, metadata);
   }

   protected void createStreams(Socket socket, Map metadata) throws IOException
   {

      String serializationType = "java"; // hardcoding to default to java serialization

      if(metadata != null)
      {
         String serializationTypeParam = (String) metadata.get(InvokerLocator.SERIALIZATIONTYPE);
         if(serializationTypeParam == null || serializationTypeParam.length() == 0)
         {
            serializationTypeParam = (String) metadata.get(InvokerLocator.SERIALIZATIONTYPE_CASED);
         }
         if(serializationTypeParam != null && serializationTypeParam.length() > 0)
         {
            serializationType = serializationTypeParam;
         }
      }

      out = createOutputStream(serializationType, socket);
      in = createInputStream(serializationType, socket);
   }

   protected InputStream createInputStream(String serializationType, Socket socket)
         throws IOException
   {
//      BufferedInputStream bin = new BufferedInputStream(socket.getInputStream());
//      ObjectInputStream oin = SerializationStreamFactory.getManagerInstance(serializationType).createInput(bin, null);
//      return oin;
      if (trace) { log.trace(this + " getting input stream from " + socket); }
      return socket.getInputStream();
   }

   protected OutputStream createOutputStream(String serializationType, Socket socket)
         throws IOException
   {
//      BufferedOutputStream bout = new BufferedOutputStream(socket.getOutputStream());
//      ObjectOutputStream oout = SerializationStreamFactory.getManagerInstance(serializationType).createOutput(bout);
//      return oout;
      if (trace) { log.trace(this + " getting output stream from " + socket); }
      return socket.getOutputStream();
   }

   public OutputStream getOutputStream()
   {
      return out;
   }

   public InputStream getInputStream()
   {
      return in;
   }

   public void checkConnection() throws IOException
   {
      // Test to see if socket is alive by send ACK message
      final byte ACK = 1;

//      out.reset();
//      out.writeByte(ACK);
//      out.flush();
//      in.readByte();

      out.write(ACK);
      out.flush();
      in.read();
   }

   public String toString()
   {
      return "ClientSocketWrapper[" + getSocket() + "]";
   }
}
